@@ -34,7 +34,7 @@ impl FromStr for ObjectStoreScheme {
     fn from_str(input: &str) -> Result<Self> {
         match input {
             "s3" => Ok(ObjectStoreScheme::S3),
-            "gcs" => Ok(ObjectStoreScheme::GCS),
+            "gs" | "gcs" => Ok(ObjectStoreScheme::GCS),
             _ => Err(DataFusionError::Execution(format!(
                 "Unsupported object store scheme {}",
                 input
@@ -107,6 +107,18 @@ mod tests {
             .contains("Not able to parse hostname from url"))
     }
 
+    #[test]
+    fn gs_provider_no_host() {
+        let no_host_url = "gs:///";
+        let provider = DatafusionCliObjectStoreProvider {};
+        let err = provider
+            .get_by_url(&Url::from_str(no_host_url).unwrap())
+            .unwrap_err();
+        assert!(err
+            .to_string()
+            .contains("Not able to parse hostname from url"))
+    }
+
     #[test]
     fn gcs_provider_no_host() {
         let no_host_url = "gcs:///";
