@@ -161,8 +161,8 @@ impl Metric {
     }
 
     /// return a reference to the partition
-    pub fn partition(&self) -> &Option<usize> {
-        &self.partition
+    pub fn partition(&self) -> Option<usize> {
+        self.partition
     }
 }
 
