@@ -32,8 +32,7 @@ use crate::physical_plan::{
     Partitioning, PhysicalExpr,
 };
 use arrow::datatypes::{Field, Schema, SchemaRef};
-use arrow::error::Result as ArrowResult;
-use arrow::record_batch::RecordBatch;
+use arrow::record_batch::{RecordBatch, RecordBatchOptions};
 use log::debug;
 
 use super::expressions::{Column, PhysicalSortExpr};
@@ -196,9 +195,9 @@ impl ExecutionPlan for ProjectionExec {
         self.output_ordering.as_deref()
     }
 
-    fn maintains_input_order(&self) -> bool {
+    fn maintains_input_order(&self) -> Vec<bool> {
         // tell optimizer this operator doesn't reorder its input
-        true
+        vec![true]
     }
 
     fn equivalence_properties(&self) -> EquivalenceProperties {
@@ -318,7 +317,7 @@ fn stats_projection(
 }
 
 impl ProjectionStream {
-    fn batch_project(&self, batch: &RecordBatch) -> ArrowResult<RecordBatch> {
+    fn batch_project(&self, batch: &RecordBatch) -> Result<RecordBatch> {
         // records time on drop
         let _timer = self.baseline_metrics.elapsed_compute().timer();
         let arrays = self
@@ -328,7 +327,14 @@ impl ProjectionStream {
             .map(|r| r.map(|v| v.into_array(batch.num_rows())))
             .collect::<Result<Vec<_>>>()?;
 
-        RecordBatch::try_new(self.schema.clone(), arrays)
+        if arrays.is_empty() {
+            let options =
+                RecordBatchOptions::new().with_row_count(Some(batch.num_rows()));
+            RecordBatch::try_new_with_options(self.schema.clone(), arrays, &options)
+                .map_err(Into::into)
+        } else {
+            RecordBatch::try_new(self.schema.clone(), arrays).map_err(Into::into)
+        }
     }
 }
 
@@ -341,7 +347,7 @@ struct ProjectionStream {
 }
 
 impl Stream for ProjectionStream {
-    type Item = ArrowResult<RecordBatch>;
+    type Item = Result<RecordBatch>;
 
     fn poll_next(
         mut self: Pin<&mut Self>,
@@ -372,6 +378,7 @@ impl RecordBatchStream for ProjectionStream {
 mod tests {
 
     use super::*;
+    use crate::physical_plan::common::collect;
     use crate::physical_plan::expressions::{self, col};
     use crate::prelude::SessionContext;
     use crate::scalar::ScalarValue;
@@ -418,6 +425,22 @@ mod tests {
         Ok(())
     }
 
+    #[tokio::test]
+    async fn project_no_column() -> Result<()> {
+        let session_ctx = SessionContext::new();
+        let task_ctx = session_ctx.task_ctx();
+
+        let csv = test::scan_partitioned_csv(1)?;
+        let expected = collect(csv.execute(0, task_ctx.clone())?).await.unwrap();
+
+        let projection = ProjectionExec::try_new(vec![], csv)?;
+        let stream = projection.execute(0, task_ctx.clone())?;
+        let output = collect(stream).await.unwrap();
+        assert_eq!(output.len(), expected.len());
+
+        Ok(())
+    }
+
     #[tokio::test]
     async fn test_stats_projection_columns_only() {
         let source = Statistics {
