@@ -22,14 +22,17 @@ use crate::expr_rewriter::{
     normalize_cols, rewrite_sort_cols_by_aggs,
 };
 use crate::type_coercion::binary::comparison_coercion;
-use crate::utils::{columnize_expr, compare_sort_expr, exprlist_to_fields, from_plan};
+use crate::utils::{
+    columnize_expr, compare_sort_expr, ensure_any_column_reference_is_unambiguous,
+    exprlist_to_fields, from_plan,
+};
 use crate::{and, binary_expr, Operator};
 use crate::{
     logical_plan::{
         Aggregate, Analyze, CrossJoin, Distinct, EmptyRelation, Explain, Filter, Join,
         JoinConstraint, JoinType, Limit, LogicalPlan, Partitioning, PlanType, Prepare,
         Projection, Repartition, Sort, SubqueryAlias, TableScan, ToStringifiedPlan,
-        Union, Values, Window,
+        Union, Unnest, Values, Window,
     },
     utils::{
         can_hash, expand_qualified_wildcard, expand_wildcard,
@@ -502,6 +505,25 @@ impl LogicalPlanBuilder {
             ));
         }
 
+        let filter = if let Some(expr) = filter {
+            // ambiguous check
+            ensure_any_column_reference_is_unambiguous(
+                &expr,
+                &[self.schema(), right.schema()],
+            )?;
+
+            // normalize all columns in expression
+            let using_columns = expr.to_columns()?;
+            let filter = normalize_col_with_schemas(
+                expr,
+                &[self.schema(), right.schema()],
+                &[using_columns],
+            )?;
+            Some(filter)
+        } else {
+            None
+        };
+
         let (left_keys, right_keys): (Vec<Result<Column>>, Vec<Result<Column>>) =
             join_keys
                 .0
@@ -745,6 +767,7 @@ impl LogicalPlanBuilder {
                 plan: Arc::new(self.plan),
                 stringified_plans,
                 schema,
+                logical_optimization_succeeded: false,
             })))
         }
     }
@@ -890,6 +913,11 @@ impl LogicalPlanBuilder {
             null_equals_null: false,
         })))
     }
+
+    /// Unnest the given column.
+    pub fn unnest_column(self, column: impl Into<Column>) -> Result<Self> {
+        Ok(Self::from(unnest(self.plan, column.into())?))
+    }
 }
 
 /// Creates a schema for a join operation.
@@ -1016,7 +1044,7 @@ pub fn union(left_plan: LogicalPlan, right_plan: LogicalPlan) -> Result<LogicalP
             match plan {
                 LogicalPlan::Projection(Projection { expr, input, .. }) => {
                     Ok(Arc::new(project_with_column_index(
-                        expr.to_vec(),
+                        expr,
                         input,
                         Arc::new(union_schema.clone()),
                     )?))
@@ -1173,6 +1201,52 @@ impl TableSource for LogicalTableSource {
     }
 }
 
+/// Create an unnest plan.
+pub fn unnest(input: LogicalPlan, column: Column) -> Result<LogicalPlan> {
+    let unnest_field = input.schema().field_from_column(&column)?;
+
+    // Extract the type of the nested field in the list.
+    let unnested_field = match unnest_field.data_type() {
+        DataType::List(field)
+        | DataType::FixedSizeList(field, _)
+        | DataType::LargeList(field) => DFField::new(
+            unnest_field.qualifier().map(String::as_str),
+            unnest_field.name(),
+            field.data_type().clone(),
+            unnest_field.is_nullable(),
+        ),
+        _ => {
+            // If the unnest field is not a list type return the input plan.
+            return Ok(input);
+        }
+    };
+
+    // Update the schema with the unnest column type changed to contain the nested type.
+    let input_schema = input.schema();
+    let fields = input_schema
+        .fields()
+        .iter()
+        .map(|f| {
+            if f == unnest_field {
+                unnested_field.clone()
+            } else {
+                f.clone()
+            }
+        })
+        .collect::<Vec<_>>();
+
+    let schema = Arc::new(DFSchema::new_with_metadata(
+        fields,
+        input_schema.metadata().clone(),
+    )?);
+
+    Ok(LogicalPlan::Unnest(Unnest {
+        input: Arc::new(input),
+        column: unnested_field.qualified_column(),
+        schema,
+    }))
+}
+
 #[cfg(test)]
 mod tests {
     use crate::{expr, expr_fn::exists};
@@ -1558,4 +1632,77 @@ mod tests {
 
         Ok(())
     }
+
+    #[test]
+    fn plan_builder_unnest() -> Result<()> {
+        // Unnesting a simple column should return the child plan.
+        let plan = nested_table_scan("test_table")?
+            .unnest_column("scalar")?
+            .build()?;
+
+        let expected = "TableScan: test_table";
+        assert_eq!(expected, format!("{plan:?}"));
+
+        // Unnesting the strings list.
+        let plan = nested_table_scan("test_table")?
+            .unnest_column("strings")?
+            .build()?;
+
+        let expected = "\
+        Unnest: test_table.strings\
+        \n  TableScan: test_table";
+        assert_eq!(expected, format!("{plan:?}"));
+
+        // Check unnested field is a scalar
+        let field = plan
+            .schema()
+            .field_with_name(Some("test_table"), "strings")
+            .unwrap();
+        assert_eq!(&DataType::Utf8, field.data_type());
+
+        // Unnesting multiple fields.
+        let plan = nested_table_scan("test_table")?
+            .unnest_column("strings")?
+            .unnest_column("structs")?
+            .build()?;
+
+        let expected = "\
+        Unnest: test_table.structs\
+        \n  Unnest: test_table.strings\
+        \n    TableScan: test_table";
+        assert_eq!(expected, format!("{plan:?}"));
+
+        // Check unnested struct list field should be a struct.
+        let field = plan
+            .schema()
+            .field_with_name(Some("test_table"), "structs")
+            .unwrap();
+        assert!(matches!(field.data_type(), DataType::Struct(_)));
+
+        // Unnesting missing column should fail.
+        let plan = nested_table_scan("test_table")?.unnest_column("missing");
+        assert!(plan.is_err());
+
+        Ok(())
+    }
+
+    fn nested_table_scan(table_name: &str) -> Result<LogicalPlanBuilder> {
+        // Create a schema with a scalar field, a list of strings, and a list of structs.
+        let struct_field = Box::new(Field::new(
+            "item",
+            DataType::Struct(vec![
+                Field::new("a", DataType::UInt32, false),
+                Field::new("b", DataType::UInt32, false),
+            ]),
+            false,
+        ));
+        let string_field = Box::new(Field::new("item", DataType::Utf8, false));
+        let schema = Schema::new(vec![
+            Field::new("scalar", DataType::UInt32, false),
+            Field::new("strings", DataType::List(string_field), false),
+            Field::new("structs", DataType::List(struct_field), false),
+        ]);
+
+        table_scan(Some(table_name), &schema, None)
+    }
 }
