@@ -15,12 +15,14 @@
 // specific language governing permissions and limitations
 // under the License.
 
-use crate::expr::BinaryExpr;
-use crate::expr_rewriter::{ExprRewritable, ExprRewriter};
+use crate::expr_rewriter::rewrite_expr;
+use crate::expr_visitor::inspect_expr_pre;
+use crate::expr_visitor::{ExprVisitable, ExpressionVisitor, Recursion};
 ///! Logical plan types
 use crate::logical_plan::builder::validate_unique_names;
 use crate::logical_plan::display::{GraphvizVisitor, IndentVisitor};
 use crate::logical_plan::extension::UserDefinedLogicalNode;
+use crate::logical_plan::plan;
 use crate::utils::{
     self, exprlist_to_fields, from_plan, grouping_set_expr_count,
     grouping_set_to_exprlist,
@@ -118,6 +120,12 @@ pub enum LogicalPlan {
     SetVariable(SetVariable),
     /// Prepare a statement
     Prepare(Prepare),
+    /// Insert / Update / Delete
+    Dml(DmlStatement),
+    /// Describe the schema of table
+    DescribeTable(DescribeTable),
+    /// Unnest a column that contains a nested list type.
+    Unnest(Unnest),
 }
 
 impl LogicalPlan {
@@ -158,6 +166,11 @@ impl LogicalPlan {
             LogicalPlan::DropTable(DropTable { schema, .. }) => schema,
             LogicalPlan::DropView(DropView { schema, .. }) => schema,
             LogicalPlan::SetVariable(SetVariable { schema, .. }) => schema,
+            LogicalPlan::DescribeTable(DescribeTable { dummy_schema, .. }) => {
+                dummy_schema
+            }
+            LogicalPlan::Dml(DmlStatement { table_schema, .. }) => table_schema,
+            LogicalPlan::Unnest(Unnest { schema, .. }) => schema,
         }
     }
 
@@ -167,10 +180,10 @@ impl LogicalPlan {
             LogicalPlan::TableScan(TableScan {
                 projected_schema, ..
             }) => vec![projected_schema],
-            LogicalPlan::Values(Values { schema, .. }) => vec![schema],
             LogicalPlan::Window(Window { input, schema, .. })
             | LogicalPlan::Projection(Projection { input, schema, .. })
-            | LogicalPlan::Aggregate(Aggregate { input, schema, .. }) => {
+            | LogicalPlan::Aggregate(Aggregate { input, schema, .. })
+            | LogicalPlan::Unnest(Unnest { input, schema, .. }) => {
                 let mut schemas = input.all_schemas();
                 schemas.insert(0, schema);
                 schemas
@@ -192,19 +205,16 @@ impl LogicalPlan {
                 schemas
             }
             LogicalPlan::Subquery(Subquery { subquery, .. }) => subquery.all_schemas(),
-            LogicalPlan::SubqueryAlias(SubqueryAlias { schema, .. }) => {
-                vec![schema]
-            }
-            LogicalPlan::Union(Union { schema, .. }) => {
-                vec![schema]
-            }
             LogicalPlan::Extension(extension) => vec![extension.node.schema()],
             LogicalPlan::Explain(Explain { schema, .. })
             | LogicalPlan::Analyze(Analyze { schema, .. })
             | LogicalPlan::EmptyRelation(EmptyRelation { schema, .. })
             | LogicalPlan::CreateExternalTable(CreateExternalTable { schema, .. })
             | LogicalPlan::CreateCatalogSchema(CreateCatalogSchema { schema, .. })
-            | LogicalPlan::CreateCatalog(CreateCatalog { schema, .. }) => {
+            | LogicalPlan::CreateCatalog(CreateCatalog { schema, .. })
+            | LogicalPlan::Values(Values { schema, .. })
+            | LogicalPlan::SubqueryAlias(SubqueryAlias { schema, .. })
+            | LogicalPlan::Union(Union { schema, .. }) => {
                 vec![schema]
             }
             LogicalPlan::Limit(Limit { input, .. })
@@ -217,7 +227,9 @@ impl LogicalPlan {
             | LogicalPlan::Prepare(Prepare { input, .. }) => input.all_schemas(),
             LogicalPlan::DropTable(_)
             | LogicalPlan::DropView(_)
+            | LogicalPlan::DescribeTable(_)
             | LogicalPlan::SetVariable(_) => vec![],
+            LogicalPlan::Dml(DmlStatement { table_schema, .. }) => vec![table_schema],
         }
     }
 
@@ -233,42 +245,74 @@ impl LogicalPlan {
     /// logical plan node. This does not include expressions in any
     /// children
     pub fn expressions(self: &LogicalPlan) -> Vec<Expr> {
+        let mut exprs = vec![];
+        self.inspect_expressions(|e| {
+            exprs.push(e.clone());
+            Ok(()) as Result<(), DataFusionError>
+        })
+        // closure always returns OK
+        .unwrap();
+        exprs
+    }
+
+    /// Calls `f` on all expressions (non-recursively) in the current
+    /// logical plan node. This does not include expressions in any
+    /// children.
+    pub fn inspect_expressions<F, E>(self: &LogicalPlan, mut f: F) -> Result<(), E>
+    where
+        F: FnMut(&Expr) -> Result<(), E>,
+    {
         match self {
-            LogicalPlan::Projection(Projection { expr, .. }) => expr.clone(),
-            LogicalPlan::Values(Values { values, .. }) => {
-                values.iter().flatten().cloned().collect()
+            LogicalPlan::Projection(Projection { expr, .. }) => {
+                expr.iter().try_for_each(f)
             }
-            LogicalPlan::Filter(Filter { predicate, .. }) => vec![predicate.clone()],
+            LogicalPlan::Values(Values { values, .. }) => {
+                values.iter().flatten().try_for_each(f)
+            }
+            LogicalPlan::Filter(Filter { predicate, .. }) => f(predicate),
             LogicalPlan::Repartition(Repartition {
                 partitioning_scheme,
                 ..
             }) => match partitioning_scheme {
-                Partitioning::Hash(expr, _) => expr.clone(),
-                Partitioning::DistributeBy(expr) => expr.clone(),
-                Partitioning::RoundRobinBatch(_) => vec![],
+                Partitioning::Hash(expr, _) => expr.iter().try_for_each(f),
+                Partitioning::DistributeBy(expr) => expr.iter().try_for_each(f),
+                Partitioning::RoundRobinBatch(_) => Ok(()),
             },
-            LogicalPlan::Window(Window { window_expr, .. }) => window_expr.clone(),
+            LogicalPlan::Window(Window { window_expr, .. }) => {
+                window_expr.iter().try_for_each(f)
+            }
             LogicalPlan::Aggregate(Aggregate {
                 group_expr,
                 aggr_expr,
                 ..
-            }) => group_expr.iter().chain(aggr_expr.iter()).cloned().collect(),
+            }) => group_expr.iter().chain(aggr_expr.iter()).try_for_each(f),
             // There are two part of expression for join, equijoin(on) and non-equijoin(filter).
             // 1. the first part is `on.len()` equijoin expressions, and the struct of each expr is `left-on = right-on`.
             // 2. the second part is non-equijoin(filter).
-            LogicalPlan::Join(Join { on, filter, .. }) => on
-                .iter()
-                .map(|(l, r)| Expr::eq(l.clone(), r.clone()))
-                .chain(
-                    filter
-                        .as_ref()
-                        .map(|expr| vec![expr.clone()])
-                        .unwrap_or_default(),
-                )
-                .collect(),
-            LogicalPlan::Sort(Sort { expr, .. }) => expr.clone(),
-            LogicalPlan::Extension(extension) => extension.node.expressions(),
-            LogicalPlan::TableScan(TableScan { filters, .. }) => filters.clone(),
+            LogicalPlan::Join(Join { on, filter, .. }) => {
+                on.iter()
+                    // it not ideal to create an expr here to analyze them, but could cache it on the Join itself
+                    .map(|(l, r)| Expr::eq(l.clone(), r.clone()))
+                    .try_for_each(|e| f(&e))?;
+
+                if let Some(filter) = filter.as_ref() {
+                    f(filter)
+                } else {
+                    Ok(())
+                }
+            }
+            LogicalPlan::Sort(Sort { expr, .. }) => expr.iter().try_for_each(f),
+            LogicalPlan::Extension(extension) => {
+                // would be nice to avoid this copy -- maybe can
+                // update extension to just observer Exprs
+                extension.node.expressions().iter().try_for_each(f)
+            }
+            LogicalPlan::TableScan(TableScan { filters, .. }) => {
+                filters.iter().try_for_each(f)
+            }
+            LogicalPlan::Unnest(Unnest { column, .. }) => {
+                f(&Expr::Column(column.clone()))
+            }
             // plans without expressions
             LogicalPlan::EmptyRelation(_)
             | LogicalPlan::Subquery(_)
@@ -287,9 +331,9 @@ impl LogicalPlan {
             | LogicalPlan::Explain(_)
             | LogicalPlan::Union(_)
             | LogicalPlan::Distinct(_)
-            | LogicalPlan::Prepare(_) => {
-                vec![]
-            }
+            | LogicalPlan::Dml(_)
+            | LogicalPlan::DescribeTable(_)
+            | LogicalPlan::Prepare(_) => Ok(()),
         }
     }
 
@@ -315,11 +359,13 @@ impl LogicalPlan {
             LogicalPlan::Distinct(Distinct { input }) => vec![input],
             LogicalPlan::Explain(explain) => vec![&explain.plan],
             LogicalPlan::Analyze(analyze) => vec![&analyze.input],
+            LogicalPlan::Dml(write) => vec![&write.input],
             LogicalPlan::CreateMemoryTable(CreateMemoryTable { input, .. })
             | LogicalPlan::CreateView(CreateView { input, .. })
             | LogicalPlan::Prepare(Prepare { input, .. }) => {
                 vec![input]
             }
+            LogicalPlan::Unnest(Unnest { input, .. }) => vec![input],
             // plans without inputs
             LogicalPlan::TableScan { .. }
             | LogicalPlan::EmptyRelation { .. }
@@ -329,7 +375,8 @@ impl LogicalPlan {
             | LogicalPlan::CreateCatalog(_)
             | LogicalPlan::DropTable(_)
             | LogicalPlan::SetVariable(_)
-            | LogicalPlan::DropView(_) => vec![],
+            | LogicalPlan::DropView(_)
+            | LogicalPlan::DescribeTable(_) => vec![],
         }
     }
 
@@ -376,7 +423,8 @@ impl LogicalPlan {
         from_plan(self, &self.expressions(), inputs)
     }
 
-    /// Convert a prepare logical plan into its inner logical plan with all params replaced with their corresponding values
+    /// Convert a prepared [`LogicalPlan`] into its inner logical plan
+    /// with all params replaced with their corresponding values
     pub fn with_param_values(
         self,
         param_values: Vec<ScalarValue>,
@@ -459,7 +507,7 @@ pub trait PlanVisitor {
 }
 
 impl LogicalPlan {
-    /// returns all inputs in the logical plan. Returns Ok(true) if
+    /// Visits all inputs in the logical plan. Returns Ok(true) if
     /// all nodes were visited, and Ok(false) if any call to
     /// `pre_visit` or `post_visit` returned Ok(false) and may have
     /// cut short the recursion
@@ -471,11 +519,12 @@ impl LogicalPlan {
             return Ok(false);
         }
 
+        // Now visit any subqueries in expressions
+        self.visit_subqueries(visitor)?;
+
         let recurse = match self {
-            LogicalPlan::Projection(Projection { .. }) => {
-                self.visit_all_inputs(visitor)?
-            }
-            LogicalPlan::Filter(Filter { .. }) => self.visit_all_inputs(visitor)?,
+            LogicalPlan::Projection(Projection { input, .. }) => input.accept(visitor)?,
+            LogicalPlan::Filter(Filter { input, .. }) => input.accept(visitor)?,
             LogicalPlan::Repartition(Repartition { input, .. }) => {
                 input.accept(visitor)?
             }
@@ -515,6 +564,8 @@ impl LogicalPlan {
             }
             LogicalPlan::Explain(explain) => explain.plan.accept(visitor)?,
             LogicalPlan::Analyze(analyze) => analyze.input.accept(visitor)?,
+            LogicalPlan::Dml(write) => write.input.accept(visitor)?,
+            LogicalPlan::Unnest(Unnest { input, .. }) => input.accept(visitor)?,
             // plans without inputs
             LogicalPlan::TableScan { .. }
             | LogicalPlan::EmptyRelation(_)
@@ -524,7 +575,8 @@ impl LogicalPlan {
             | LogicalPlan::CreateCatalog(_)
             | LogicalPlan::DropTable(_)
             | LogicalPlan::SetVariable(_)
-            | LogicalPlan::DropView(_) => true,
+            | LogicalPlan::DropView(_)
+            | LogicalPlan::DescribeTable(_) => true,
         };
         if !recurse {
             return Ok(false);
@@ -537,52 +589,36 @@ impl LogicalPlan {
         Ok(true)
     }
 
-    /// Visit all inputs, including subqueries
-    pub fn visit_all_inputs<V>(&self, visitor: &mut V) -> Result<bool, V::Error>
+    /// applies visitor to any subqueries in the plan
+    fn visit_subqueries<V>(&self, v: &mut V) -> Result<bool, V::Error>
     where
         V: PlanVisitor,
     {
-        for input in self.all_inputs() {
-            if !input.accept(visitor)? {
-                return Ok(false);
-            }
-        }
-
+        self.inspect_expressions(|expr| {
+            // recursively look for subqueries
+            inspect_expr_pre(expr, |expr| {
+                match expr {
+                    Expr::Exists { subquery, .. }
+                    | Expr::InSubquery { subquery, .. }
+                    | Expr::ScalarSubquery(subquery) => {
+                        // use a synthetic plan so the visitor sees a
+                        // LogicalPlan::Subquery (even though it is
+                        // actually a Subquery alias)
+                        let synthetic_plan = LogicalPlan::Subquery(subquery.clone());
+                        synthetic_plan.accept(v)?;
+                    }
+                    _ => {}
+                }
+                Ok(())
+            })
+        })?;
+        // continue recursion
         Ok(true)
     }
 
-    /// Get all plan inputs, including subqueries from expressions
-    fn all_inputs(&self) -> Vec<Arc<LogicalPlan>> {
-        let mut inputs = vec![];
-        for expr in self.expressions() {
-            Self::collect_subqueries(&expr, &mut inputs);
-        }
-        for input in self.inputs() {
-            inputs.push(Arc::new(input.clone()));
-        }
-        inputs
-    }
-
-    fn collect_subqueries(expr: &Expr, sub: &mut Vec<Arc<LogicalPlan>>) {
-        match expr {
-            Expr::BinaryExpr(BinaryExpr { left, right, .. }) => {
-                Self::collect_subqueries(left, sub);
-                Self::collect_subqueries(right, sub);
-            }
-            Expr::Exists { subquery, .. } => {
-                sub.push(Arc::new(LogicalPlan::Subquery(subquery.clone())));
-            }
-            Expr::InSubquery { subquery, .. } => {
-                sub.push(Arc::new(LogicalPlan::Subquery(subquery.clone())));
-            }
-            Expr::ScalarSubquery(subquery) => {
-                sub.push(Arc::new(LogicalPlan::Subquery(subquery.clone())));
-            }
-            _ => {}
-        }
-    }
-
-    /// Return a logical plan with all placeholders/params (e.g $1 $2, ...) replaced with corresponding values provided in the prams_values
+    /// Return a logical plan with all placeholders/params (e.g $1 $2,
+    /// ...) replaced with corresponding values provided in the
+    /// params_values
     pub fn replace_params_with_values(
         &self,
         param_values: &Vec<ScalarValue>,
@@ -603,18 +639,80 @@ impl LogicalPlan {
         Ok(new_plan)
     }
 
-    /// Return an Expr with all placeholders replaced with their corresponding values provided in the prams_values
-    fn replace_placeholders_with_values(
-        expr: Expr,
-        param_values: &Vec<ScalarValue>,
-    ) -> Result<Expr, DataFusionError> {
-        struct PlaceholderReplacer<'a> {
-            param_values: &'a Vec<ScalarValue>,
+    /// Walk the logical plan, find any `PlaceHolder` tokens, and return a map of their IDs and DataTypes
+    pub fn get_parameter_types(
+        &self,
+    ) -> Result<HashMap<String, Option<DataType>>, DataFusionError> {
+        struct ParamTypeVisitor {
+            param_types: HashMap<String, Option<DataType>>,
         }
 
-        impl<'a> ExprRewriter for PlaceholderReplacer<'a> {
-            fn mutate(&mut self, expr: Expr) -> Result<Expr, DataFusionError> {
-                if let Expr::Placeholder { id, data_type } = &expr {
+        struct ExprParamTypeVisitor {
+            param_types: HashMap<String, Option<DataType>>,
+        }
+
+        impl ExpressionVisitor for ExprParamTypeVisitor {
+            fn pre_visit(
+                mut self,
+                expr: &Expr,
+            ) -> datafusion_common::Result<Recursion<Self>>
+            where
+                Self: ExpressionVisitor,
+            {
+                if let Expr::Placeholder { id, data_type } = expr {
+                    let prev = self.param_types.get(id);
+                    match (prev, data_type) {
+                        (Some(Some(prev)), Some(dt)) => {
+                            if prev != dt {
+                                Err(DataFusionError::Plan(format!(
+                                    "Conflicting types for {id}"
+                                )))?;
+                            }
+                        }
+                        (_, Some(dt)) => {
+                            let _ = self.param_types.insert(id.clone(), Some(dt.clone()));
+                        }
+                        _ => {}
+                    }
+                }
+                Ok(Recursion::Continue(self))
+            }
+        }
+
+        impl PlanVisitor for ParamTypeVisitor {
+            type Error = DataFusionError;
+
+            fn pre_visit(&mut self, plan: &LogicalPlan) -> Result<bool, Self::Error> {
+                let mut param_types = HashMap::new();
+                plan.inspect_expressions(|expr| {
+                    let mut visitor = ExprParamTypeVisitor {
+                        param_types: Default::default(),
+                    };
+                    visitor = expr.accept(visitor)?;
+                    param_types.extend(visitor.param_types);
+                    Ok(()) as Result<(), DataFusionError>
+                })?;
+                self.param_types.extend(param_types);
+                Ok(true)
+            }
+        }
+
+        let mut visitor = ParamTypeVisitor {
+            param_types: Default::default(),
+        };
+        self.accept(&mut visitor)?;
+        Ok(visitor.param_types)
+    }
+
+    /// Return an Expr with all placeholders replaced with their
+    /// corresponding values provided in the params_values
+    fn replace_placeholders_with_values(
+        expr: Expr,
+        param_values: &[ScalarValue],
+    ) -> Result<Expr, DataFusionError> {
+        rewrite_expr(expr, |expr| {
+            match &expr {
+                Expr::Placeholder { id, data_type } => {
                     // convert id (in format $1, $2, ..) to idx (0, 1, ..)
                     let idx = id[1..].parse::<usize>().map_err(|e| {
                         DataFusionError::Internal(format!(
@@ -622,13 +720,13 @@ impl LogicalPlan {
                         ))
                     })? - 1;
                     // value at the idx-th position in param_values should be the value for the placeholder
-                    let value = self.param_values.get(idx).ok_or_else(|| {
+                    let value = param_values.get(idx).ok_or_else(|| {
                         DataFusionError::Internal(format!(
                             "No value found for placeholder with id {id}"
                         ))
                     })?;
                     // check if the data type of the value matches the data type of the placeholder
-                    if value.get_datatype() != *data_type {
+                    if Some(value.get_datatype()) != *data_type {
                         return Err(DataFusionError::Internal(format!(
                             "Placeholder value type mismatch: expected {:?}, got {:?}",
                             data_type,
@@ -637,13 +735,17 @@ impl LogicalPlan {
                     }
                     // Replace the placeholder with the value
                     Ok(Expr::Literal(value.clone()))
-                } else {
-                    Ok(expr)
                 }
+                Expr::ScalarSubquery(qry) => {
+                    let subquery = Arc::new(
+                        qry.subquery
+                            .replace_params_with_values(&param_values.to_vec())?,
+                    );
+                    Ok(Expr::ScalarSubquery(plan::Subquery { subquery }))
+                }
+                _ => Ok(expr),
             }
-        }
-
-        expr.rewrite(&mut PlaceholderReplacer { param_values })
+        })
     }
 }
 
@@ -915,6 +1017,9 @@ impl LogicalPlan {
                         }
                         Ok(())
                     }
+                    LogicalPlan::Dml(DmlStatement { table_name, op, .. }) => {
+                        write!(f, "Dml: op=[{op}] table=[{table_name}]")
+                    }
                     LogicalPlan::Filter(Filter {
                         predicate: ref expr,
                         ..
@@ -1080,6 +1185,12 @@ impl LogicalPlan {
                     }) => {
                         write!(f, "Prepare: {name:?} {data_types:?} ")
                     }
+                    LogicalPlan::DescribeTable(DescribeTable { .. }) => {
+                        write!(f, "DescribeTable")
+                    }
+                    LogicalPlan::Unnest(Unnest { column, .. }) => {
+                        write!(f, "Unnest: {column}")
+                    }
                 }
             }
         }
@@ -1195,7 +1306,8 @@ pub struct DropView {
     pub schema: DFSchemaRef,
 }
 
-/// Set a Variable's value -- value in [`ConfigOptions`]
+/// Set a Variable's value -- value in
+/// [`ConfigOptions`](datafusion_common::config::ConfigOptions)
 #[derive(Clone)]
 pub struct SetVariable {
     /// The variable name
@@ -1490,6 +1602,38 @@ pub struct CreateExternalTable {
     pub options: HashMap<String, String>,
 }
 
+#[derive(Clone)]
+pub enum WriteOp {
+    Insert,
+    Delete,
+    Update,
+    Ctas,
+}
+
+impl Display for WriteOp {
+    fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
+        match self {
+            WriteOp::Insert => write!(f, "Insert"),
+            WriteOp::Delete => write!(f, "Delete"),
+            WriteOp::Update => write!(f, "Update"),
+            WriteOp::Ctas => write!(f, "Ctas"),
+        }
+    }
+}
+
+/// The operator that modifies the content of a database (adapted from substrait WriteRel)
+#[derive(Clone)]
+pub struct DmlStatement {
+    /// The table name
+    pub table_name: OwnedTableReference,
+    /// The schema of the table (must align with Rel input)
+    pub table_schema: DFSchemaRef,
+    /// The type of operation to perform
+    pub op: WriteOp,
+    /// The relation that determines the tuples to add/remove/modify the schema must match with table_schema
+    pub input: Arc<LogicalPlan>,
+}
+
 /// Prepare a statement but do not execute it. Prepare statements can have 0 or more
 /// `Expr::Placeholder` expressions that are filled in during execution
 #[derive(Clone)]
@@ -1502,6 +1646,15 @@ pub struct Prepare {
     pub input: Arc<LogicalPlan>,
 }
 
+/// Describe the schema of table
+#[derive(Clone)]
+pub struct DescribeTable {
+    /// Table schema
+    pub schema: Arc<Schema>,
+    /// Dummy schema
+    pub dummy_schema: DFSchemaRef,
+}
+
 /// Produces a relation with string representations of
 /// various parts of the plan
 #[derive(Clone)]
@@ -1514,6 +1667,8 @@ pub struct Explain {
     pub stringified_plans: Vec<StringifiedPlan>,
     /// The output schema of the explain (2 columns of text)
     pub schema: DFSchemaRef,
+    /// Used by physical planner to check if should proceed with planning
+    pub logical_optimization_succeeded: bool,
 }
 
 /// Runs the actual plan, and then prints the physical plan with
@@ -1833,11 +1988,22 @@ pub trait ToStringifiedPlan {
     fn to_stringified(&self, plan_type: PlanType) -> StringifiedPlan;
 }
 
+/// Unnest a column that contains a nested list type.
+#[derive(Debug, Clone)]
+pub struct Unnest {
+    /// The incoming logical plan
+    pub input: Arc<LogicalPlan>,
+    /// The column to unnest
+    pub column: Column,
+    /// The output schema, containing the unnested field column.
+    pub schema: DFSchemaRef,
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
     use crate::logical_plan::table_scan;
-    use crate::{col, in_subquery, lit};
+    use crate::{col, exists, in_subquery, lit};
     use arrow::datatypes::{DataType, Field, Schema};
     use datafusion_common::DFSchema;
     use datafusion_common::Result;
@@ -1891,6 +2057,26 @@ mod tests {
         Ok(())
     }
 
+    #[test]
+    fn test_display_subquery_alias() -> Result<()> {
+        let plan1 = table_scan(Some("employee_csv"), &employee_schema(), Some(vec![3]))?
+            .build()?;
+        let plan1 = Arc::new(plan1);
+
+        let plan =
+            table_scan(Some("employee_csv"), &employee_schema(), Some(vec![0, 3]))?
+                .project(vec![col("id"), exists(plan1).alias("exists")])?
+                .build();
+
+        let expected = "Projection: employee_csv.id, EXISTS (<subquery>) AS exists\
+        \n  Subquery:\
+        \n    TableScan: employee_csv projection=[state]\
+        \n  TableScan: employee_csv projection=[id, state]";
+
+        assert_eq!(expected, format!("{}", plan?.display_indent()));
+        Ok(())
+    }
+
     #[test]
     fn test_display_graphviz() -> Result<()> {
         let plan = display_plan()?;
@@ -1932,10 +2118,7 @@ mod tests {
     impl PlanVisitor for OkVisitor {
         type Error = String;
 
-        fn pre_visit(
-            &mut self,
-            plan: &LogicalPlan,
-        ) -> std::result::Result<bool, Self::Error> {
+        fn pre_visit(&mut self, plan: &LogicalPlan) -> Result<bool, Self::Error> {
             let s = match plan {
                 LogicalPlan::Projection { .. } => "pre_visit Projection",
                 LogicalPlan::Filter { .. } => "pre_visit Filter",
@@ -1947,10 +2130,7 @@ mod tests {
             Ok(true)
         }
 
-        fn post_visit(
-            &mut self,
-            plan: &LogicalPlan,
-        ) -> std::result::Result<bool, Self::Error> {
+        fn post_visit(&mut self, plan: &LogicalPlan) -> Result<bool, Self::Error> {
             let s = match plan {
                 LogicalPlan::Projection { .. } => "post_visit Projection",
                 LogicalPlan::Filter { .. } => "post_visit Filter",
@@ -2017,20 +2197,14 @@ mod tests {
     impl PlanVisitor for StoppingVisitor {
         type Error = String;
 
-        fn pre_visit(
-            &mut self,
-            plan: &LogicalPlan,
-        ) -> std::result::Result<bool, Self::Error> {
+        fn pre_visit(&mut self, plan: &LogicalPlan) -> Result<bool, Self::Error> {
             if self.return_false_from_pre_in.dec() {
                 return Ok(false);
             }
             self.inner.pre_visit(plan)
         }
 
-        fn post_visit(
-            &mut self,
-            plan: &LogicalPlan,
-        ) -> std::result::Result<bool, Self::Error> {
+        fn post_visit(&mut self, plan: &LogicalPlan) -> Result<bool, Self::Error> {
             if self.return_false_from_post_in.dec() {
                 return Ok(false);
             }
@@ -2090,10 +2264,7 @@ mod tests {
     impl PlanVisitor for ErrorVisitor {
         type Error = String;
 
-        fn pre_visit(
-            &mut self,
-            plan: &LogicalPlan,
-        ) -> std::result::Result<bool, Self::Error> {
+        fn pre_visit(&mut self, plan: &LogicalPlan) -> Result<bool, Self::Error> {
             if self.return_error_from_pre_in.dec() {
                 return Err("Error in pre_visit".into());
             }
@@ -2101,10 +2272,7 @@ mod tests {
             self.inner.pre_visit(plan)
         }
 
-        fn post_visit(
-            &mut self,
-            plan: &LogicalPlan,
-        ) -> std::result::Result<bool, Self::Error> {
+        fn post_visit(&mut self, plan: &LogicalPlan) -> Result<bool, Self::Error> {
             if self.return_error_from_post_in.dec() {
                 return Err("Error in post_visit".into());
             }
