@@ -16,11 +16,14 @@
 // under the License.
 
 use crate::window::partition_evaluator::PartitionEvaluator;
+use crate::window::window_frame_state::WindowFrameContext;
 use crate::{PhysicalExpr, PhysicalSortExpr};
+use arrow::array::{new_empty_array, ArrayRef};
 use arrow::compute::kernels::partition::lexicographical_partition_ranges;
 use arrow::compute::kernels::sort::SortColumn;
+use arrow::compute::{concat, SortOptions};
+use arrow::datatypes::Field;
 use arrow::record_batch::RecordBatch;
-use arrow::{array::ArrayRef, datatypes::Field};
 use arrow_schema::DataType;
 use datafusion_common::{reverse_sort_options, DataFusionError, Result, ScalarValue};
 use datafusion_expr::{Accumulator, WindowFrame};
@@ -64,7 +67,8 @@ pub trait WindowExpr: Send + Sync + Debug {
     /// evaluate the window function values against the batch
     fn evaluate(&self, batch: &RecordBatch) -> Result<ArrayRef>;
 
-    /// evaluate the window function values against the batch
+    /// Evaluate the window function against the batch. This function facilitates
+    /// stateful, bounded-memory implementations.
     fn evaluate_stateful(
         &self,
         _partition_batches: &PartitionBatches,
@@ -139,6 +143,136 @@ pub trait WindowExpr: Send + Sync + Debug {
     fn get_reverse_expr(&self) -> Option<Arc<dyn WindowExpr>>;
 }
 
+/// Trait for different `AggregateWindowExpr`s (`PlainAggregateWindowExpr`, `SlidingAggregateWindowExpr`)
+pub trait AggregateWindowExpr: WindowExpr {
+    /// Get the accumulator for the window expression. Note that distinct
+    /// window expressions may return distinct accumulators; e.g. sliding
+    /// (non-sliding) expressions will return sliding (normal) accumulators.
+    fn get_accumulator(&self) -> Result<Box<dyn Accumulator>>;
+
+    /// Given current range and the last range, calculates the accumulator
+    /// result for the range of interest.
+    fn get_aggregate_result_inside_range(
+        &self,
+        last_range: &Range<usize>,
+        cur_range: &Range<usize>,
+        value_slice: &[ArrayRef],
+        accumulator: &mut Box<dyn Accumulator>,
+    ) -> Result<ScalarValue>;
+
+    /// Evaluates the window function against the batch.
+    fn aggregate_evaluate(&self, batch: &RecordBatch) -> Result<ArrayRef> {
+        let mut window_frame_ctx = WindowFrameContext::new(self.get_window_frame());
+        let mut accumulator = self.get_accumulator()?;
+        let mut last_range = Range { start: 0, end: 0 };
+        let mut idx = 0;
+        self.get_result_column(
+            &mut accumulator,
+            batch,
+            &mut window_frame_ctx,
+            &mut last_range,
+            &mut idx,
+            false,
+        )
+    }
+
+    /// Statefully evaluates the window function against the batch. Maintains
+    /// state so that it can work incrementally over multiple chunks.
+    fn aggregate_evaluate_stateful(
+        &self,
+        partition_batches: &PartitionBatches,
+        window_agg_state: &mut PartitionWindowAggStates,
+    ) -> Result<()> {
+        let field = self.field()?;
+        let out_type = field.data_type();
+        for (partition_row, partition_batch_state) in partition_batches.iter() {
+            if !window_agg_state.contains_key(partition_row) {
+                let accumulator = self.get_accumulator()?;
+                window_agg_state.insert(
+                    partition_row.clone(),
+                    WindowState {
+                        state: WindowAggState::new(out_type)?,
+                        window_fn: WindowFn::Aggregate(accumulator),
+                    },
+                );
+            };
+            let window_state =
+                window_agg_state.get_mut(partition_row).ok_or_else(|| {
+                    DataFusionError::Execution("Cannot find state".to_string())
+                })?;
+            let accumulator = match &mut window_state.window_fn {
+                WindowFn::Aggregate(accumulator) => accumulator,
+                _ => unreachable!(),
+            };
+            let mut state = &mut window_state.state;
+
+            let record_batch = &partition_batch_state.record_batch;
+            let mut window_frame_ctx = WindowFrameContext::new(self.get_window_frame());
+            let out_col = self.get_result_column(
+                accumulator,
+                record_batch,
+                &mut window_frame_ctx,
+                &mut state.window_frame_range,
+                &mut state.last_calculated_index,
+                !partition_batch_state.is_end,
+            )?;
+            state.is_end = partition_batch_state.is_end;
+            state.out_col = concat(&[&state.out_col, &out_col])?;
+            state.n_row_result_missing =
+                record_batch.num_rows() - state.last_calculated_index;
+        }
+        Ok(())
+    }
+
+    /// Calculates the window expression result for the given record batch.
+    /// Assumes that `record_batch` belongs to a single partition.
+    fn get_result_column(
+        &self,
+        accumulator: &mut Box<dyn Accumulator>,
+        record_batch: &RecordBatch,
+        window_frame_ctx: &mut WindowFrameContext,
+        last_range: &mut Range<usize>,
+        idx: &mut usize,
+        not_end: bool,
+    ) -> Result<ArrayRef> {
+        let (values, order_bys) = self.get_values_orderbys(record_batch)?;
+        // We iterate on each row to perform a running calculation.
+        let length = values[0].len();
+        let sort_options: Vec<SortOptions> =
+            self.order_by().iter().map(|o| o.options).collect();
+        let mut row_wise_results: Vec<ScalarValue> = vec![];
+        while *idx < length {
+            let cur_range = window_frame_ctx.calculate_range(
+                &order_bys,
+                &sort_options,
+                length,
+                *idx,
+                last_range,
+            )?;
+            // Exit if the range extends all the way:
+            if cur_range.end == length && not_end {
+                break;
+            }
+            let value = self.get_aggregate_result_inside_range(
+                last_range,
+                &cur_range,
+                &values,
+                accumulator,
+            )?;
+            last_range.clone_from(&cur_range);
+            row_wise_results.push(value);
+            *idx += 1;
+        }
+        if row_wise_results.is_empty() {
+            let field = self.field()?;
+            let out_type = field.data_type();
+            Ok(new_empty_array(out_type))
+        } else {
+            ScalarValue::iter_to_array(row_wise_results.into_iter())
+        }
+    }
+}
+
 /// Reverses the ORDER BY expression, which is useful during equivalent window
 /// expression construction. For instance, 'ORDER BY a ASC, NULLS LAST' turns into
 /// 'ORDER BY a DESC, NULLS FIRST'.
@@ -158,8 +292,7 @@ pub enum WindowFn {
     Aggregate(Box<dyn Accumulator>),
 }
 
-/// State for RANK(percent_rank, rank, dense_rank)
-/// builtin window function
+/// State for the RANK(percent_rank, rank, dense_rank) built-in window function.
 #[derive(Debug, Clone, Default)]
 pub struct RankState {
     /// The last values for rank as these values change, we increase n_rank
@@ -170,15 +303,33 @@ pub struct RankState {
     pub n_rank: usize,
 }
 
-/// State for 'ROW_NUMBER' builtin window function
+/// State for the 'ROW_NUMBER' built-in window function.
 #[derive(Debug, Clone, Default)]
 pub struct NumRowsState {
     pub n_rows: usize,
 }
 
-#[derive(Debug, Clone, Default)]
+/// Tag to differentiate special use cases of the NTH_VALUE built-in window function.
+#[derive(Debug, Copy, Clone)]
+pub enum NthValueKind {
+    First,
+    Last,
+    Nth(u32),
+}
+
+#[derive(Debug, Clone)]
 pub struct NthValueState {
     pub range: Range<usize>,
+    // In certain cases, we can finalize the result early. Consider this usage:
+    // ```
+    //  FIRST_VALUE(increasing_col) OVER window AS my_first_value
+    //  WINDOW (ORDER BY ts ASC ROWS BETWEEN UNBOUNDED PRECEDING AND 1 FOLLOWING) AS window
+    // ```
+    // The result will always be the first entry in the table. We can store such
+    // early-finalizing results and then just reuse them as necessary. This opens
+    // opportunities to prune our datasets.
+    pub finalized_result: Option<ScalarValue>,
+    pub kind: NthValueKind,
 }
 
 #[derive(Debug, Clone, Default)]
@@ -195,15 +346,6 @@ pub enum BuiltinWindowState {
     #[default]
     Default,
 }
-#[derive(Debug)]
-pub enum WindowFunctionState {
-    /// Different Aggregate functions may have different state definitions
-    /// In [Accumulator] trait, [fn state(&self) -> Result<Vec<ScalarValue>>] implementation
-    /// dictates that.
-    AggregateState(Vec<ScalarValue>),
-    /// BuiltinWindowState
-    BuiltinWindowState(BuiltinWindowState),
-}
 
 #[derive(Debug)]
 pub struct WindowAggState {
@@ -213,9 +355,6 @@ pub struct WindowAggState {
     pub last_calculated_index: usize,
     /// The offset of the deleted row number
     pub offset_pruned_rows: usize,
-    /// State of the window function, required to calculate its result
-    // For instance, for ROW_NUMBER we keep the row index counter to generate correct result
-    pub window_function_state: WindowFunctionState,
     /// Stores the results calculated by window frame
     pub out_col: ArrayRef,
     /// Keeps track of how many rows should be generated to be in sync with input record_batch.
@@ -250,16 +389,12 @@ pub type PartitionWindowAggStates = IndexMap<PartitionKey, WindowState>;
 pub type PartitionBatches = IndexMap<PartitionKey, PartitionBatchState>;
 
 impl WindowAggState {
-    pub fn new(
-        out_type: &DataType,
-        window_function_state: WindowFunctionState,
-    ) -> Result<Self> {
+    pub fn new(out_type: &DataType) -> Result<Self> {
         let empty_out_col = ScalarValue::try_from(out_type)?.to_array_of_size(0);
         Ok(Self {
             window_frame_range: Range { start: 0, end: 0 },
             last_calculated_index: 0,
             offset_pruned_rows: 0,
-            window_function_state,
             out_col: empty_out_col,
             n_row_result_missing: 0,
             is_end: false,
