@@ -223,6 +223,11 @@ impl TryFrom<&DataType> for protobuf::arrow_type::ArrowTypeEnum {
                     "Proto serialization error: The Map data type is not yet supported".to_owned()
                 ))
             }
+            DataType::RunEndEncoded(_, _) => {
+                return Err(Error::General(
+                    "Proto serialization error: The RunEndEncoded data type is not yet supported".to_owned()
+                ))
+            }
         };
 
         Ok(res)
@@ -892,8 +897,17 @@ impl TryFrom<&Expr> for protobuf::LogicalExprNode {
                         .collect::<Result<Vec<_>, Self::Error>>()?,
                 })),
             },
-            Expr::Placeholder{ id, data_type } => Self {
-                expr_type: Some(ExprType::Placeholder(PlaceholderNode { id: id.clone(), data_type: Some(data_type.try_into()?) })),
+            Expr::Placeholder{ id, data_type } => {
+                let data_type = match data_type {
+                    Some(data_type) => Some(data_type.try_into()?),
+                    None => None,
+                };
+                Self {
+                    expr_type: Some(ExprType::Placeholder(PlaceholderNode {
+                        id: id.clone(),
+                        data_type,
+                    })),
+                }
             },
 
             Expr::QualifiedWildcard { .. } | Expr::TryCast { .. } =>
