@@ -59,7 +59,15 @@ impl From<&protobuf::PhysicalColumn> for Column {
     }
 }
 
-pub(crate) fn parse_physical_expr(
+/// Parses a physical expression from a protobuf.
+///
+/// # Arguments
+///
+/// * `proto` - Input proto with physical expression node
+/// * `registry` - A registry knows how to build logical expressions out of user-defined function' names
+/// * `input_schema` - The Arrow schema for the input, used for determining expression data types
+///                    when performing type coercion.
+pub fn parse_physical_expr(
     proto: &protobuf::PhysicalExprNode,
     registry: &dyn FunctionRegistry,
     input_schema: &Schema,
@@ -76,24 +84,34 @@ pub(crate) fn parse_physical_expr(
         }
         ExprType::Literal(scalar) => Arc::new(Literal::new(scalar.try_into()?)),
         ExprType::BinaryExpr(binary_expr) => Arc::new(BinaryExpr::new(
-            parse_required_physical_box_expr(
-                &binary_expr.l,
+            parse_required_physical_expr(
+                binary_expr.l.as_deref(),
                 registry,
                 "left",
                 input_schema,
             )?,
             logical_plan::from_proto::from_proto_binary_op(&binary_expr.op)?,
-            parse_required_physical_box_expr(
-                &binary_expr.r,
+            parse_required_physical_expr(
+                binary_expr.r.as_deref(),
                 registry,
                 "right",
                 input_schema,
             )?,
         )),
         ExprType::DateTimeIntervalExpr(expr) => Arc::new(DateTimeIntervalExpr::try_new(
-            parse_required_physical_box_expr(&expr.l, registry, "left", input_schema)?,
+            parse_required_physical_expr(
+                expr.l.as_deref(),
+                registry,
+                "left",
+                input_schema,
+            )?,
             logical_plan::from_proto::from_proto_binary_op(&expr.op)?,
-            parse_required_physical_box_expr(&expr.r, registry, "right", input_schema)?,
+            parse_required_physical_expr(
+                expr.r.as_deref(),
+                registry,
+                "right",
+                input_schema,
+            )?,
             input_schema,
         )?),
         ExprType::AggregateExpr(_) => {
@@ -111,23 +129,43 @@ pub(crate) fn parse_physical_expr(
                 "Cannot convert sort expr node to physical expression".to_owned(),
             ));
         }
-        ExprType::IsNullExpr(e) => Arc::new(IsNullExpr::new(
-            parse_required_physical_box_expr(&e.expr, registry, "expr", input_schema)?,
-        )),
-        ExprType::IsNotNullExpr(e) => Arc::new(IsNotNullExpr::new(
-            parse_required_physical_box_expr(&e.expr, registry, "expr", input_schema)?,
-        )),
-        ExprType::NotExpr(e) => Arc::new(NotExpr::new(parse_required_physical_box_expr(
-            &e.expr,
+        ExprType::IsNullExpr(e) => {
+            Arc::new(IsNullExpr::new(parse_required_physical_expr(
+                e.expr.as_deref(),
+                registry,
+                "expr",
+                input_schema,
+            )?))
+        }
+        ExprType::IsNotNullExpr(e) => {
+            Arc::new(IsNotNullExpr::new(parse_required_physical_expr(
+                e.expr.as_deref(),
+                registry,
+                "expr",
+                input_schema,
+            )?))
+        }
+        ExprType::NotExpr(e) => Arc::new(NotExpr::new(parse_required_physical_expr(
+            e.expr.as_deref(),
             registry,
             "expr",
             input_schema,
         )?)),
-        ExprType::Negative(e) => Arc::new(NegativeExpr::new(
-            parse_required_physical_box_expr(&e.expr, registry, "expr", input_schema)?,
-        )),
+        ExprType::Negative(e) => {
+            Arc::new(NegativeExpr::new(parse_required_physical_expr(
+                e.expr.as_deref(),
+                registry,
+                "expr",
+                input_schema,
+            )?))
+        }
         ExprType::InList(e) => Arc::new(InListExpr::new(
-            parse_required_physical_box_expr(&e.expr, registry, "expr", input_schema)?,
+            parse_required_physical_expr(
+                e.expr.as_deref(),
+                registry,
+                "expr",
+                input_schema,
+            )?,
             e.list
                 .iter()
                 .map(|x| parse_physical_expr(x, registry, input_schema))
@@ -165,12 +203,22 @@ pub(crate) fn parse_physical_expr(
                 .transpose()?,
         )?),
         ExprType::Cast(e) => Arc::new(CastExpr::new(
-            parse_required_physical_box_expr(&e.expr, registry, "expr", input_schema)?,
+            parse_required_physical_expr(
+                e.expr.as_deref(),
+                registry,
+                "expr",
+                input_schema,
+            )?,
             convert_required!(e.arrow_type)?,
             DEFAULT_DATAFUSION_CAST_OPTIONS,
         )),
         ExprType::TryCast(e) => Arc::new(TryCastExpr::new(
-            parse_required_physical_box_expr(&e.expr, registry, "expr", input_schema)?,
+            parse_required_physical_expr(
+                e.expr.as_deref(),
+                registry,
+                "expr",
+                input_schema,
+            )?,
             convert_required!(e.arrow_type)?,
         )),
         ExprType::ScalarFunction(e) => {
@@ -221,14 +269,14 @@ pub(crate) fn parse_physical_expr(
         ExprType::LikeExpr(like_expr) => Arc::new(LikeExpr::new(
             like_expr.negated,
             like_expr.case_insensitive,
-            parse_required_physical_box_expr(
-                &like_expr.expr,
+            parse_required_physical_expr(
+                like_expr.expr.as_deref(),
                 registry,
                 "expr",
                 input_schema,
             )?,
-            parse_required_physical_box_expr(
-                &like_expr.pattern,
+            parse_required_physical_expr(
+                like_expr.pattern.as_deref(),
                 registry,
                 "pattern",
                 input_schema,
@@ -239,28 +287,13 @@ pub(crate) fn parse_physical_expr(
     Ok(pexpr)
 }
 
-fn parse_required_physical_box_expr(
-    expr: &Option<Box<protobuf::PhysicalExprNode>>,
-    registry: &dyn FunctionRegistry,
-    field: &str,
-    input_schema: &Schema,
-) -> Result<Arc<dyn PhysicalExpr>, DataFusionError> {
-    expr.as_ref()
-        .map(|e| parse_physical_expr(e.as_ref(), registry, input_schema))
-        .transpose()?
-        .ok_or_else(|| {
-            DataFusionError::Internal(format!("Missing required field {field:?}"))
-        })
-}
-
 fn parse_required_physical_expr(
     expr: Option<&protobuf::PhysicalExprNode>,
     registry: &dyn FunctionRegistry,
     field: &str,
     input_schema: &Schema,
 ) -> Result<Arc<dyn PhysicalExpr>, DataFusionError> {
-    expr.as_ref()
-        .map(|e| parse_physical_expr(e, registry, input_schema))
+    expr.map(|e| parse_physical_expr(e, registry, input_schema))
         .transpose()?
         .ok_or_else(|| {
             DataFusionError::Internal(format!("Missing required field {field:?}"))
