@@ -16,39 +16,70 @@
 // under the License.
 
 use crate::parser::{
-    CreateExternalTable, DFParser, DescribeTable, Statement as DFStatement,
+    CreateExternalTable, DFParser, DescribeTableStmt, Statement as DFStatement,
 };
 use crate::planner::{
-    object_name_to_qualifier, object_name_to_table_reference, ContextProvider,
-    PlannerContext, SqlToRel,
+    object_name_to_qualifier, ContextProvider, PlannerContext, SqlToRel,
 };
+use crate::utils::normalize_ident;
 use arrow_schema::DataType;
 use datafusion_common::parsers::CompressionTypeVariant;
 use datafusion_common::{
-    DFSchema, DFSchemaRef, DataFusionError, OwnedTableReference, Result, TableReference,
-    ToDFSchema,
+    Column, DFField, DFSchema, DFSchemaRef, DataFusionError, ExprSchema,
+    OwnedTableReference, Result, TableReference, ToDFSchema,
 };
+use datafusion_expr::expr_rewriter::normalize_col_with_schemas;
+use datafusion_expr::logical_plan::builder::project;
 use datafusion_expr::logical_plan::{Analyze, Prepare};
+use datafusion_expr::utils::expr_to_columns;
 use datafusion_expr::{
     cast, col, CreateCatalog, CreateCatalogSchema,
     CreateExternalTable as PlanCreateExternalTable, CreateMemoryTable, CreateView,
-    DropTable, DropView, Explain, LogicalPlan, LogicalPlanBuilder, PlanType, SetVariable,
-    ToStringifiedPlan,
+    DescribeTable, DmlStatement, DropTable, DropView, Explain, ExprSchemable, Filter,
+    LogicalPlan, LogicalPlanBuilder, PlanType, SetVariable, ToStringifiedPlan, WriteOp,
 };
+use sqlparser::ast;
 use sqlparser::ast::{
-    Expr as SQLExpr, Ident, ObjectName, ObjectType, ShowCreateObject,
-    ShowStatementFilter, Statement, UnaryOperator, Value,
+    Assignment, Expr as SQLExpr, Expr, Ident, ObjectName, ObjectType, Query, SchemaName,
+    SetExpr, ShowCreateObject, ShowStatementFilter, Statement, TableFactor,
+    TableWithJoins, UnaryOperator, Value,
 };
 use sqlparser::parser::ParserError::ParserError;
+use std::collections::{BTreeMap, HashMap, HashSet};
 use std::sync::Arc;
 
+fn ident_to_string(ident: &Ident) -> String {
+    normalize_ident(ident.to_owned())
+}
+
+fn object_name_to_string(object_name: &ObjectName) -> String {
+    object_name
+        .0
+        .iter()
+        .map(ident_to_string)
+        .collect::<Vec<String>>()
+        .join(".")
+}
+
+fn get_schema_name(schema_name: &SchemaName) -> String {
+    match schema_name {
+        SchemaName::Simple(schema_name) => object_name_to_string(schema_name),
+        SchemaName::UnnamedAuthorization(auth) => ident_to_string(auth),
+        SchemaName::NamedAuthorization(schema_name, auth) => format!(
+            "{}.{}",
+            object_name_to_string(schema_name),
+            ident_to_string(auth)
+        ),
+    }
+}
+
 impl<'a, S: ContextProvider> SqlToRel<'a, S> {
     /// Generate a logical plan from an DataFusion SQL statement
     pub fn statement_to_plan(&self, statement: DFStatement) -> Result<LogicalPlan> {
         match statement {
             DFStatement::CreateExternalTable(s) => self.external_table_to_plan(s),
             DFStatement::Statement(s) => self.sql_statement_to_plan(*s),
-            DFStatement::DescribeTable(s) => self.describe_table_to_plan(s),
+            DFStatement::DescribeTableStmt(s) => self.describe_table_to_plan(s),
         }
     }
 
@@ -126,7 +157,7 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
                 };
 
                 Ok(LogicalPlan::CreateMemoryTable(CreateMemoryTable {
-                    name: object_name_to_table_reference(name)?,
+                    name: self.object_name_to_table_reference(name)?,
                     input: Arc::new(plan),
                     if_not_exists,
                     or_replace,
@@ -144,7 +175,7 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
                 plan = self.apply_expr_alias(plan, columns)?;
 
                 Ok(LogicalPlan::CreateView(CreateView {
-                    name: object_name_to_table_reference(name)?,
+                    name: self.object_name_to_table_reference(name)?,
                     input: Arc::new(plan),
                     or_replace,
                     definition: sql,
@@ -164,7 +195,7 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
                 schema_name,
                 if_not_exists,
             } => Ok(LogicalPlan::CreateCatalogSchema(CreateCatalogSchema {
-                schema_name: schema_name.to_string(),
+                schema_name: get_schema_name(&schema_name),
                 if_not_exists,
                 schema: Arc::new(DFSchema::empty()),
             })),
@@ -173,7 +204,7 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
                 if_not_exists,
                 ..
             } => Ok(LogicalPlan::CreateCatalog(CreateCatalog {
-                catalog_name: db_name.to_string(),
+                catalog_name: object_name_to_string(&db_name),
                 if_not_exists,
                 schema: Arc::new(DFSchema::empty()),
             })),
@@ -189,7 +220,7 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
                 // nor do we support multiple object names
                 let name = match names.len() {
                     0 => Err(ParserError("Missing table name.".to_string()).into()),
-                    1 => object_name_to_table_reference(names.pop().unwrap()),
+                    1 => self.object_name_to_table_reference(names.pop().unwrap()),
                     _ => {
                         Err(ParserError("Multiple objects not supported".to_string())
                             .into())
@@ -234,7 +265,7 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
                     &mut planner_context,
                 )?;
                 Ok(LogicalPlan::Prepare(Prepare {
-                    name: name.to_string(),
+                    name: ident_to_string(&name),
                     data_types,
                     input: Arc::new(plan),
                 }))
@@ -253,6 +284,93 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
                 table_name,
                 filter,
             } => self.show_columns_to_plan(extended, full, table_name, filter),
+
+            Statement::Insert {
+                or,
+                into,
+                table_name,
+                columns,
+                overwrite,
+                source,
+                partitioned,
+                after_columns,
+                table,
+                on,
+                returning,
+            } => {
+                if or.is_some() {
+                    Err(DataFusionError::Plan(
+                        "Inserts with or clauses not supported".to_owned(),
+                    ))?;
+                }
+                if overwrite {
+                    Err(DataFusionError::Plan(
+                        "Insert overwrite is not supported".to_owned(),
+                    ))?;
+                }
+                if partitioned.is_some() {
+                    Err(DataFusionError::Plan(
+                        "Partitioned inserts not yet supported".to_owned(),
+                    ))?;
+                }
+                if !after_columns.is_empty() {
+                    Err(DataFusionError::Plan(
+                        "After-columns clause not supported".to_owned(),
+                    ))?;
+                }
+                if table {
+                    Err(DataFusionError::Plan(
+                        "Table clause not supported".to_owned(),
+                    ))?;
+                }
+                if on.is_some() {
+                    Err(DataFusionError::Plan(
+                        "Insert-on clause not supported".to_owned(),
+                    ))?;
+                }
+                if returning.is_some() {
+                    Err(DataFusionError::Plan(
+                        "Insert-returning clause not yet supported".to_owned(),
+                    ))?;
+                }
+                let _ = into; // optional keyword doesn't change behavior
+                self.insert_to_plan(table_name, columns, source)
+            }
+
+            Statement::Update {
+                table,
+                assignments,
+                from,
+                selection,
+                returning,
+            } => {
+                if returning.is_some() {
+                    Err(DataFusionError::Plan(
+                        "Update-returning clause not yet supported".to_owned(),
+                    ))?;
+                }
+                self.update_to_plan(table, assignments, from, selection)
+            }
+
+            Statement::Delete {
+                table_name,
+                using,
+                selection,
+                returning,
+            } => {
+                if using.is_some() {
+                    Err(DataFusionError::Plan(
+                        "Using clause not supported".to_owned(),
+                    ))?;
+                }
+                if returning.is_some() {
+                    Err(DataFusionError::Plan(
+                        "Delete-returning clause not yet supported".to_owned(),
+                    ))?;
+                }
+                self.delete_to_plan(table_name, selection)
+            }
+
             _ => Err(DataFusionError::NotImplemented(format!(
                 "Unsupported SQL statement: {sql:?}"
             ))),
@@ -288,30 +406,23 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
         }
     }
 
-    fn describe_table_to_plan(&self, statement: DescribeTable) -> Result<LogicalPlan> {
-        let DescribeTable { table_name } = statement;
+    fn describe_table_to_plan(
+        &self,
+        statement: DescribeTableStmt,
+    ) -> Result<LogicalPlan> {
+        let DescribeTableStmt { table_name } = statement;
+        let table_ref = self.object_name_to_table_reference(table_name)?;
 
-        let where_clause = object_name_to_qualifier(&table_name);
-        let table_ref = object_name_to_table_reference(table_name)?;
-
-        // check if table_name exists
-        let _ = self
+        let table_source = self
             .schema_provider
             .get_table_provider((&table_ref).into())?;
 
-        if self.has_table("information_schema", "tables") {
-            let sql = format!(
-                "SELECT column_name, data_type, is_nullable \
-                                FROM information_schema.columns WHERE {where_clause};"
-            );
-            let mut rewrite = DFParser::parse_sql(&sql)?;
-            self.statement_to_plan(rewrite.pop_front().unwrap())
-        } else {
-            Err(DataFusionError::Plan(
-                "DESCRIBE TABLE is not supported unless information_schema is enabled"
-                    .to_string(),
-            ))
-        }
+        let schema = table_source.schema();
+
+        Ok(LogicalPlan::DescribeTable(DescribeTable {
+            schema,
+            dummy_schema: DFSchemaRef::new(DFSchema::empty()),
+        }))
     }
 
     /// Generate a logical plan from a CREATE EXTERNAL TABLE statement
@@ -396,12 +507,13 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
                 plan,
                 stringified_plans,
                 schema,
+                logical_optimization_succeeded: false,
             }))
         }
     }
 
     fn show_variable_to_plan(&self, variable: &[Ident]) -> Result<LogicalPlan> {
-        let variable = ObjectName(variable.to_vec()).to_string();
+        let variable = object_name_to_string(&ObjectName(variable.to_vec()));
 
         if !self.has_table("information_schema", "df_settings") {
             return Err(DataFusionError::Plan(
@@ -451,7 +563,7 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
             ));
         }
 
-        let variable = variable.to_string();
+        let variable = object_name_to_string(variable);
         let mut variable_lower = variable.to_lowercase();
 
         if variable_lower == "timezone" || variable_lower == "time.zone" {
@@ -461,7 +573,7 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
 
         // parse value string from Expr
         let value_string = match &value[0] {
-            SQLExpr::Identifier(i) => i.to_string(),
+            SQLExpr::Identifier(i) => ident_to_string(i),
             SQLExpr::Value(v) => match v {
                 Value::SingleQuotedString(s) => s.to_string(),
                 Value::DollarQuotedString(s) => s.to_string(),
@@ -505,6 +617,255 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
         }))
     }
 
+    fn delete_to_plan(
+        &self,
+        table_factor: TableFactor,
+        predicate_expr: Option<Expr>,
+    ) -> Result<LogicalPlan> {
+        let table_name = match &table_factor {
+            TableFactor::Table { name, .. } => name.clone(),
+            _ => Err(DataFusionError::Plan(
+                "Cannot delete from non-table relations!".to_string(),
+            ))?,
+        };
+
+        // Do a table lookup to verify the table exists
+        let table_ref = self.object_name_to_table_reference(table_name.clone())?;
+        let provider = self
+            .schema_provider
+            .get_table_provider((&table_ref).into())?;
+        let schema = (*provider.schema()).clone();
+        let schema = DFSchema::try_from(schema)?;
+        let scan =
+            LogicalPlanBuilder::scan(object_name_to_string(&table_name), provider, None)?
+                .build()?;
+        let mut planner_context = PlannerContext::new();
+
+        let source = match predicate_expr {
+            None => scan,
+            Some(predicate_expr) => {
+                let filter_expr =
+                    self.sql_to_expr(predicate_expr, &schema, &mut planner_context)?;
+                let schema = Arc::new(schema.clone());
+                let mut using_columns = HashSet::new();
+                expr_to_columns(&filter_expr, &mut using_columns)?;
+                let filter_expr = normalize_col_with_schemas(
+                    filter_expr,
+                    &[&schema],
+                    &[using_columns],
+                )?;
+                LogicalPlan::Filter(Filter::try_new(filter_expr, Arc::new(scan))?)
+            }
+        };
+
+        let plan = LogicalPlan::Dml(DmlStatement {
+            table_name: table_ref,
+            table_schema: schema.into(),
+            op: WriteOp::Delete,
+            input: Arc::new(source),
+        });
+        Ok(plan)
+    }
+
+    fn update_to_plan(
+        &self,
+        table: TableWithJoins,
+        assignments: Vec<Assignment>,
+        from: Option<TableWithJoins>,
+        predicate_expr: Option<Expr>,
+    ) -> Result<LogicalPlan> {
+        let table_name = match &table.relation {
+            TableFactor::Table { name, .. } => name.clone(),
+            _ => Err(DataFusionError::Plan(
+                "Cannot update non-table relation!".to_string(),
+            ))?,
+        };
+
+        // Do a table lookup to verify the table exists
+        let table_name = self.object_name_to_table_reference(table_name)?;
+        let provider = self
+            .schema_provider
+            .get_table_provider((&table_name).into())?;
+        let arrow_schema = (*provider.schema()).clone();
+        let table_schema = Arc::new(DFSchema::try_from(arrow_schema)?);
+        let values = table_schema.fields().iter().map(|f| {
+            (
+                f.name().clone(),
+                ast::Expr::Identifier(ast::Ident::from(f.name().as_str())),
+            )
+        });
+
+        // Overwrite with assignment expressions
+        let mut planner_context = PlannerContext::new();
+        let mut assign_map = assignments
+            .iter()
+            .map(|assign| {
+                let col_name: &Ident = assign
+                    .id
+                    .iter()
+                    .last()
+                    .ok_or(DataFusionError::Plan("Empty column id".to_string()))?;
+                // Validate that the assignment target column exists
+                table_schema.field_with_unqualified_name(&col_name.value)?;
+                Ok((col_name.value.clone(), assign.value.clone()))
+            })
+            .collect::<Result<HashMap<String, Expr>>>()?;
+
+        let values = values
+            .into_iter()
+            .map(|(k, v)| {
+                let val = assign_map.remove(&k).unwrap_or(v);
+                (k, val)
+            })
+            .collect::<Vec<_>>();
+
+        // Build scan
+        let from = from.unwrap_or(table);
+        let scan = self.plan_from_tables(vec![from], &mut planner_context)?;
+
+        // Filter
+        let source = match predicate_expr {
+            None => scan,
+            Some(predicate_expr) => {
+                let filter_expr = self.sql_to_expr(
+                    predicate_expr,
+                    &table_schema,
+                    &mut planner_context,
+                )?;
+                let mut using_columns = HashSet::new();
+                expr_to_columns(&filter_expr, &mut using_columns)?;
+                let filter_expr = normalize_col_with_schemas(
+                    filter_expr,
+                    &[&table_schema],
+                    &[using_columns],
+                )?;
+                LogicalPlan::Filter(Filter::try_new(filter_expr, Arc::new(scan))?)
+            }
+        };
+
+        // Projection
+        let mut exprs = vec![];
+        for (col_name, expr) in values.into_iter() {
+            let expr = self.sql_to_expr(expr, &table_schema, &mut planner_context)?;
+            let expr = match expr {
+                datafusion_expr::Expr::Placeholder {
+                    ref id,
+                    ref data_type,
+                } => match data_type {
+                    None => {
+                        let dt = table_schema.data_type(&Column::from_name(&col_name))?;
+                        datafusion_expr::Expr::Placeholder {
+                            id: id.clone(),
+                            data_type: Some(dt.clone()),
+                        }
+                    }
+                    Some(_) => expr,
+                },
+                _ => expr,
+            };
+            let expr = expr.alias(col_name);
+            exprs.push(expr);
+        }
+        let source = project(source, exprs)?;
+
+        let plan = LogicalPlan::Dml(DmlStatement {
+            table_name,
+            table_schema,
+            op: WriteOp::Update,
+            input: Arc::new(source),
+        });
+        Ok(plan)
+    }
+
+    fn insert_to_plan(
+        &self,
+        table_name: ObjectName,
+        columns: Vec<Ident>,
+        source: Box<Query>,
+    ) -> Result<LogicalPlan> {
+        // Do a table lookup to verify the table exists
+        let table_name = self.object_name_to_table_reference(table_name)?;
+        let provider = self
+            .schema_provider
+            .get_table_provider((&table_name).into())?;
+        let arrow_schema = (*provider.schema()).clone();
+        let table_schema = DFSchema::try_from(arrow_schema)?;
+
+        let fields = if columns.is_empty() {
+            // Empty means we're inserting into all columns of the table
+            table_schema.fields().clone()
+        } else {
+            let fields = columns
+                .iter()
+                .map(|c| {
+                    Ok(table_schema
+                        .field_with_unqualified_name(&normalize_ident(c.clone()))?
+                        .clone())
+                })
+                .collect::<Result<Vec<DFField>>>()?;
+            // Validate no duplicate fields
+            let table_schema =
+                DFSchema::new_with_metadata(fields, table_schema.metadata().clone())?;
+            table_schema.fields().clone()
+        };
+
+        // infer types for Values clause... other types should be resolvable the regular way
+        let mut prepare_param_data_types = BTreeMap::new();
+        if let SetExpr::Values(ast::Values { rows, .. }) = (*source.body).clone() {
+            for row in rows.iter() {
+                for (idx, val) in row.iter().enumerate() {
+                    if let ast::Expr::Value(Value::Placeholder(name)) = val {
+                        let name =
+                            name.replace('$', "").parse::<usize>().map_err(|_| {
+                                DataFusionError::Plan(format!(
+                                    "Can't parse placeholder: {name}"
+                                ))
+                            })? - 1;
+                        let field = fields.get(idx).ok_or_else(|| {
+                            DataFusionError::Plan(format!(
+                                "Placeholder ${} refers to a non existent column",
+                                idx + 1
+                            ))
+                        })?;
+                        let dt = field.field().data_type().clone();
+                        let _ = prepare_param_data_types.insert(name, dt);
+                    }
+                }
+            }
+        }
+        let prepare_param_data_types = prepare_param_data_types.into_values().collect();
+
+        // Projection
+        let mut planner_context =
+            PlannerContext::new_with_prepare_param_data_types(prepare_param_data_types);
+        let source = self.query_to_plan(*source, &mut planner_context)?;
+        if fields.len() != source.schema().fields().len() {
+            Err(DataFusionError::Plan(
+                "Column count doesn't match insert query!".to_owned(),
+            ))?;
+        }
+        let exprs = fields
+            .iter()
+            .zip(source.schema().fields().iter())
+            .map(|(target_field, source_field)| {
+                let expr =
+                    datafusion_expr::Expr::Column(source_field.unqualified_column())
+                        .cast_to(target_field.data_type(), source.schema())?
+                        .alias(target_field.name());
+                Ok(expr)
+            })
+            .collect::<Result<Vec<datafusion_expr::Expr>>>()?;
+        let source = project(source, exprs)?;
+
+        let plan = LogicalPlan::Dml(DmlStatement {
+            table_name,
+            table_schema: Arc::new(table_schema),
+            op: WriteOp::Insert,
+            input: Arc::new(source),
+        });
+        Ok(plan)
+    }
+
     fn show_columns_to_plan(
         &self,
         extended: bool,
@@ -525,10 +886,13 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
             ));
         }
         // Figure out the where clause
-        let where_clause = object_name_to_qualifier(&sql_table_name);
+        let where_clause = object_name_to_qualifier(
+            &sql_table_name,
+            self.options.enable_ident_normalization,
+        );
 
         // Do a table lookup to verify the table exists
-        let table_ref = object_name_to_table_reference(sql_table_name)?;
+        let table_ref = self.object_name_to_table_reference(sql_table_name)?;
         let _ = self
             .schema_provider
             .get_table_provider((&table_ref).into())?;
@@ -560,10 +924,13 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
             ));
         }
         // Figure out the where clause
-        let where_clause = object_name_to_qualifier(&sql_table_name);
+        let where_clause = object_name_to_qualifier(
+            &sql_table_name,
+            self.options.enable_ident_normalization,
+        );
 
         // Do a table lookup to verify the table exists
-        let table_ref = object_name_to_table_reference(sql_table_name)?;
+        let table_ref = self.object_name_to_table_reference(sql_table_name)?;
         let _ = self
             .schema_provider
             .get_table_provider((&table_ref).into())?;
@@ -579,7 +946,10 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
 
     /// Return true if there is a table provider available for "schema.table"
     fn has_table(&self, schema: &str, table: &str) -> bool {
-        let tables_reference = TableReference::Partial { schema, table };
+        let tables_reference = TableReference::Partial {
+            schema: schema.into(),
+            table: table.into(),
+        };
         self.schema_provider
             .get_table_provider(tables_reference)
             .is_ok()
